@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
  * Copyright (C) 2009 Dan Williams, <dcbw@redhat.com>
+ * Copyright (C) 2009 - 2011 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,7 +66,8 @@ test_items (const char *detail, NMSettingVPN *s_vpn, Item *items, gboolean secre
 			ASSERT (value == NULL, detail, "unexpected item '%s'", iter->name);
 		} else {
 			ASSERT (value != NULL, detail, "unexpected missing value for item %s", iter->name);
-			ASSERT (strcmp (value, iter->value) == 0, detail, "unexpected value for item %s", iter->name);
+			ASSERT (strcmp (value, iter->value) == 0, detail, "unexpected value for item %s (%s != %s",
+			        iter->name, value, iter->value);
 			expected_count++;
 		}
 	}
@@ -76,7 +78,7 @@ test_items (const char *detail, NMSettingVPN *s_vpn, Item *items, gboolean secre
 		nm_setting_vpn_foreach_data_item (s_vpn, item_count_func, &actual_count);
 
 	ASSERT (actual_count == expected_count,
-	        detail, "unexpected number of items (got %d, expected %d", actual_count, expected_count);
+	        detail, "unexpected number of items (got %d, expected %d)", actual_count, expected_count);
 }
 
 
@@ -87,7 +89,7 @@ static Item basic_items[] = {
 	{ NM_VPNC_KEY_XAUTH_USER,            "bsmith" },
 	{ NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,   NM_VPNC_PW_TYPE_SAVE },
 	{ NM_VPNC_KEY_DOMAIN,                "COMPANY" },
-	{ NM_VPNC_KEY_DHGROUP,               "2" },
+	{ NM_VPNC_KEY_DHGROUP,               "dh2" },
 	{ NM_VPNC_KEY_PERFECT_FORWARD,       NULL },
 	{ NM_VPNC_KEY_APP_VERSION,           NULL },
 	{ NM_VPNC_KEY_SINGLE_DES,            NULL },
@@ -95,6 +97,9 @@ static Item basic_items[] = {
 	{ NM_VPNC_KEY_NAT_TRAVERSAL_MODE,    NM_VPNC_NATT_MODE_CISCO },
 	{ NM_VPNC_KEY_DPD_IDLE_TIMEOUT,      "90" },
 	{ NM_VPNC_KEY_CISCO_UDP_ENCAPS_PORT, NULL },
+	{ NM_VPNC_KEY_LOCAL_PORT,            "500" },
+	{ NM_VPNC_KEY_SECRET"-flags",        "1" },
+	{ NM_VPNC_KEY_XAUTH_PASSWORD"-flags","1" },
 	{ NULL, NULL }
 };
 
@@ -262,10 +267,11 @@ remove_secrets (NMConnection *connection)
 
 #define BASIC_EXPORTED_NAME "basic-export-test.pcf"
 static void
-test_basic_export (NMVpnPluginUiInterface *plugin, const char *dir)
+test_basic_export (NMVpnPluginUiInterface *plugin, const char *dir, const char *tmpdir)
 {
 	NMConnection *connection;
 	NMConnection *reimported;
+	NMSetting *s_vpn;
 	char *path;
 	gboolean success;
 	GError *error = NULL;
@@ -274,7 +280,7 @@ test_basic_export (NMVpnPluginUiInterface *plugin, const char *dir)
 	connection = get_basic_connection ("basic-export", plugin, dir, "basic.pcf");
 	ASSERT (connection != NULL, "basic-export", "failed to import connection");
 
-	path = g_build_path ("/", dir, BASIC_EXPORTED_NAME, NULL);
+	path = g_build_path ("/", tmpdir, BASIC_EXPORTED_NAME, NULL);
 	success = nm_vpn_plugin_ui_interface_export (plugin, path, connection, &error);
 	if (!success) {
 		if (!error)
@@ -284,7 +290,7 @@ test_basic_export (NMVpnPluginUiInterface *plugin, const char *dir)
 	}
 
 	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection ("basic-export", plugin, dir, BASIC_EXPORTED_NAME);
+	reimported = get_basic_connection ("basic-export", plugin, tmpdir, BASIC_EXPORTED_NAME);
 	ret = unlink (path);
 	ASSERT (connection != NULL, "basic-export", "failed to re-import connection");
 
@@ -293,6 +299,15 @@ test_basic_export (NMVpnPluginUiInterface *plugin, const char *dir)
 	 */
 	remove_secrets (connection);
 
+	/* Since we don't export the user password, but the original connection
+	 * had one, we need to add secret flags to the re-imported connection.
+	 */
+	s_vpn = nm_connection_get_setting (reimported, NM_TYPE_SETTING_VPN);
+	nm_setting_set_secret_flags (s_vpn,
+	                             NM_VPNC_KEY_SECRET,
+	                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
+	                             NULL);
+
 	ASSERT (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT) == TRUE,
 	        "basic-export", "original and reimported connection differ");
 
@@ -303,7 +318,10 @@ test_basic_export (NMVpnPluginUiInterface *plugin, const char *dir)
 
 #define NAT_EXPORTED_NAME "nat-export-test.pcf"
 static void
-test_nat_export (NMVpnPluginUiInterface *plugin, const char *dir, const char *nat_mode)
+test_nat_export (NMVpnPluginUiInterface *plugin,
+                 const char *dir,
+                 const char *tmpdir,
+                 const char *nat_mode)
 {
 	NMConnection *connection;
 	NMSettingVPN *s_vpn;
@@ -321,7 +339,7 @@ test_nat_export (NMVpnPluginUiInterface *plugin, const char *dir, const char *na
 
 	nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_NAT_TRAVERSAL_MODE, nat_mode);
 
-	path = g_build_path ("/", dir, NAT_EXPORTED_NAME, NULL);
+	path = g_build_path ("/", tmpdir, NAT_EXPORTED_NAME, NULL);
 	success = nm_vpn_plugin_ui_interface_export (plugin, path, connection, &error);
 	if (!success) {
 		if (!error)
@@ -331,7 +349,7 @@ test_nat_export (NMVpnPluginUiInterface *plugin, const char *dir, const char *na
 	}
 
 	/* Now re-import it and compare the connections to ensure they are the same */
-	reimported = get_basic_connection ("nat-export", plugin, dir, NAT_EXPORTED_NAME);
+	reimported = get_basic_connection ("nat-export", plugin, tmpdir, NAT_EXPORTED_NAME);
 	ret = unlink (path);
 	ASSERT (connection != NULL, "nat-export", "failed to re-import connection");
 
@@ -340,6 +358,15 @@ test_nat_export (NMVpnPluginUiInterface *plugin, const char *dir, const char *na
 	 */
 	remove_secrets (connection);
 
+	/* Since we don't export the user password, but the original connection
+	 * had one, we need to add secret flags to the re-imported connection.
+	 */
+	s_vpn = nm_connection_get_setting_vpn (reimported);
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn),
+	                             NM_VPNC_KEY_SECRET,
+	                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
+	                             NULL);
+
 	ASSERT (nm_connection_compare (connection, reimported, NM_SETTING_COMPARE_FLAG_EXACT) == TRUE,
 	        "nat-export", "original and reimported connection differ");
 
@@ -519,6 +546,49 @@ test_nat_natt (NMVpnPluginUiInterface *plugin, const char *dir)
 	g_free (pcf);
 }
 
+static void
+test_nat_force_natt (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingVPN *s_vpn;
+	GError *error = NULL;
+	char *pcf;
+	const char *expected_id = "Force NAT-T";
+	const char *value;
+
+	pcf = g_build_path ("/", dir, "force-natt.pcf", NULL);
+	ASSERT (pcf != NULL,
+	        "force-natt", "failed to create pcf path");
+
+	connection = nm_vpn_plugin_ui_interface_import (plugin, pcf, &error);
+	if (error)
+		FAIL ("force-natt", "error importing %s: %s", pcf, error->message);
+	ASSERT (connection != NULL,
+	        "force-natt", "error importing %s: (unknown)", pcf);
+
+	/* Connection setting */
+	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	ASSERT (s_con != NULL,
+	        "force-natt", "missing 'connection' setting");
+
+	ASSERT (strcmp (nm_setting_connection_get_id (s_con), expected_id) == 0,
+	        "force-natt", "unexpected connection ID");
+
+	/* VPN setting */
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	ASSERT (s_vpn != NULL,
+	        "force-natt", "missing 'vpn' setting");
+
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_NAT_TRAVERSAL_MODE);
+	ASSERT (value != NULL,
+	        "force-natt", "unexpected missing value for item %s", NM_VPNC_KEY_NAT_TRAVERSAL_MODE);
+	ASSERT (strcmp (value, NM_VPNC_NATT_MODE_NATT_ALWAYS) == 0,
+	        "force-natt", "unexpected value for item %s", NM_VPNC_KEY_NAT_TRAVERSAL_MODE);
+
+	g_free (pcf);
+}
+
 static void
 test_always_ask (NMVpnPluginUiInterface *plugin, const char *dir)
 {
@@ -596,6 +666,90 @@ test_non_utf8_import (NMVpnPluginUiInterface *plugin, const char *dir)
 	g_object_unref (connection);
 }
 
+static void
+test_legacy_ike_port_0_import (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingVPN *s_vpn;
+	GError *error = NULL;
+	char *pcf;
+	const char *expected_id = "Use Legacy IKE Port (i.e. dynamic)";
+	const char *value;
+
+	pcf = g_build_path ("/", dir, "use-legacy-ike-port-0.pcf", NULL);
+	ASSERT (pcf != NULL,
+	        "use-legacy-ike-port-0", "failed to create pcf path");
+
+	connection = nm_vpn_plugin_ui_interface_import (plugin, pcf, &error);
+	if (error)
+		FAIL ("", "error importing %s: %s", pcf, error->message);
+	ASSERT (connection != NULL,
+	        "use-legacy-ike-port-0", "error importing %s: (unknown)", pcf);
+
+	/* Connection setting */
+	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	ASSERT (s_con != NULL,
+	        "use-legacy-ike-port-0", "missing 'connection' setting");
+
+	ASSERT (strcmp (nm_setting_connection_get_id (s_con), expected_id) == 0,
+	        "use-legacy-ike-port-0", "unexpected connection ID");
+
+	/* VPN setting */
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	ASSERT (s_vpn != NULL,
+	        "use-legacy-ike-port-0", "missing 'vpn' setting");
+
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_LOCAL_PORT);
+	ASSERT (value == NULL || strcmp (value, "0") == 0,
+	        "use-legacy-ike-port-0", "item %s should not be present or should be 0", NM_VPNC_KEY_LOCAL_PORT);
+
+	g_free (pcf);
+}
+
+static void
+test_legacy_ike_port_1_import (NMVpnPluginUiInterface *plugin, const char *dir)
+{
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	NMSettingVPN *s_vpn;
+	GError *error = NULL;
+	char *pcf;
+	const char *expected_id = "Don't use Legacy IKE Port (500)";
+	const char *value;
+
+	pcf = g_build_path ("/", dir, "use-legacy-ike-port-1.pcf", NULL);
+	ASSERT (pcf != NULL,
+	        "use-legacy-ike-port-1", "failed to create pcf path");
+
+	connection = nm_vpn_plugin_ui_interface_import (plugin, pcf, &error);
+	if (error)
+		FAIL ("", "error importing %s: %s", pcf, error->message);
+	ASSERT (connection != NULL,
+	        "use-legacy-ike-port-1", "error importing %s: (unknown)", pcf);
+
+	/* Connection setting */
+	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	ASSERT (s_con != NULL,
+	        "use-legacy-ike-port-1", "missing 'connection' setting");
+
+	ASSERT (strcmp (nm_setting_connection_get_id (s_con), expected_id) == 0,
+	        "use-legacy-ike-port-1", "unexpected connection ID");
+
+	/* VPN setting */
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	ASSERT (s_vpn != NULL,
+	        "use-legacy-ike-port-1", "missing 'vpn' setting");
+
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_LOCAL_PORT);
+	ASSERT (value != NULL,
+	        "use-legacy-ike-port-1", "unexpected missing value for item %s", NM_VPNC_KEY_LOCAL_PORT);
+	ASSERT (strcmp (value, "500") == 0,
+	        "use-legacy-ike-port-1", "unexpected value for item %s", NM_VPNC_KEY_LOCAL_PORT);
+
+	g_free (pcf);
+}
+
 int main (int argc, char **argv)
 {
 	GError *error = NULL;
@@ -603,8 +757,8 @@ int main (int argc, char **argv)
 	char *basename;
 	NMVpnPluginUiInterface *plugin = NULL;
 
-	if (argc != 2)
-		FAIL ("args", "usage: %s <pcf path>", argv[0]);
+	if (argc != 3)
+		FAIL ("args", "usage: %s <pcf path> <tmp path>", argv[0]);
 
 	g_type_init ();
 	bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
@@ -624,12 +778,16 @@ int main (int argc, char **argv)
 	test_no_natt (plugin, argv[1]);
 	test_nat_cisco (plugin, argv[1]);
 	test_nat_natt (plugin, argv[1]);
+	test_nat_force_natt (plugin, argv[1]);
 	test_always_ask (plugin, argv[1]);
 	test_non_utf8_import (plugin, argv[1]);
+	test_legacy_ike_port_0_import (plugin, argv[1]);
+	test_legacy_ike_port_1_import (plugin, argv[1]);
 
-	test_basic_export (plugin, argv[1]);
-	test_nat_export (plugin, argv[1], NM_VPNC_NATT_MODE_CISCO);
-	test_nat_export (plugin, argv[1], NM_VPNC_NATT_MODE_NATT);
+	test_basic_export (plugin, argv[1], argv[2]);
+	test_nat_export (plugin, argv[1], argv[2], NM_VPNC_NATT_MODE_CISCO);
+	test_nat_export (plugin, argv[1], argv[2], NM_VPNC_NATT_MODE_NATT);
+	test_nat_export (plugin, argv[1], argv[2], NM_VPNC_NATT_MODE_NATT_ALWAYS);
 
 	g_object_unref (plugin);
 
